# Develop Word2Vec Embedding
# https://machinelearningmastery.com/develop-word-embeddings-python-gensim/
# https://radimrehurek.com/gensim/auto_examples/tutorials/run_fasttext.html#sphx-glr-download-auto-examples-tutorials-run-fasttext-py
from gensim.models import Word2Vec
import sys

def load_text(filename):
		#f = open(filename, "r", encoding='utf-8')
		f = open(filename, "r", encoding='cp949')
		text = f.readlines()
		f.close()
		return text

def sent2tokens(text):
		tokens = []
		for sent in text:
			tokens.append(sent.split())
		return tokens

def Word2Vec_train(sentences):
	print(f"\Word2Vec training for sentences.\n")

	model = Word2Vec(sentences, min_count=1)	# train model
	print(model)	# summarize the loaded model

	# summarize vocabulary
	# words = list(model.wv.vocab) # Error -- gensim OLD version
	words = model.wv.index_to_key
	print(words)

	# access vector for one word
	# print(model['sentence']) # ERROR -- gensim OLD version
	print(model.wv.get_vector('sentence'))
	
	# save model
	model.save('model.bin')
	
	# load model
	new_model = Word2Vec.load('model.bin')
	print(new_model)


#from pprint import pprint as print
from gensim.models.fasttext import FastText
from gensim.test.utils import datapath

def FastText_train_textfile(filename):
	print(f"\nFastText training for corpus_file <{filename}>.")

	model = FastText(vector_size=300, min_count=10, workers=1)
	#model = FastText(vector_size=12, min_count=1, hs=1, negative=0, seed=42, workers=1, bucket=BUCKET)

	# build the vocabulary
	model.build_vocab(corpus_file=filename)

	# train the model
	model.train(corpus_file=filename, epochs=model.epochs, total_examples=model.corpus_count, total_words=model.corpus_total_words,)

	return model


def FastText_train_sentences(sentences):
	print(f"\nFastText training for sentences.")

	model = FastText(vector_size=300)
	#model = FastText(vector_size=12, min_count=1, hs=1, negative=0, seed=42, workers=1, bucket=BUCKET)

	# build the vocabulary
	model.build_vocab(sentences)

	# train the model
	model.train(sentences, total_examples=len(sentences), epochs=300)

	return model


if __name__ == "__main__":
	# training data for Word2Vec
	sentences = [['this', 'is', 'the', 'first', 'sentence', 'for', 'word2vec'],
			['this', 'is', 'the', 'second', 'sentence'],
			['yet', 'another', 'sentence'],
			['one', 'more', 'sentence'],
			['and', 'the', 'final', 'sentence']]

	print("\nWord2Vec training...")
	Word2Vec_train(sentences)


	# FastText training for corpus_file
	# Set file names for train and test data
	#corpus_file = datapath('lee_background.cor')	# gensim example file
	corpus_file = 'botchan.txt'	# file name of a big corpus text
	model = FastText_train_textfile(corpus_file)
	model.save('botchan.model')
	print(model)


	# training data -- text corpus
	sentences = ["this is the first sentence for FastText",
			"this is the second sentence",
			"yet another sentence",
			"one more sentence",
			"and the final sentence"]
	#token_list = sent2tokens(sentences)
	#Word2Vec_train(token_list)	# test for word2vec model

	filename = "D:\KCC_KMA_Word2Vec\KCCq28-KMA_euckr.txt"
	if len(sys.argv) >= 2:
			filename = sys.argv[1]
	sentences = load_text(filename)

	# FastText training for sentences
	token_list = sent2tokens(sentences)
	model = FastText_train_sentences(token_list)

	model_file = filename[:-14] + '_FastText' + '.model'
	model.save(model_file)

	# load model
	new_model = Word2Vec.load(model_file)
	print(new_model)
