# Develop Word2Vec Embedding
# https://machinelearningmastery.com/develop-word-embeddings-python-gensim/
# https://radimrehurek.com/gensim/auto_examples/tutorials/run_fasttext.html#sphx-glr-download-auto-examples-tutorials-run-fasttext-py
# C> FastText_Train_ADD.py "FastText-kowiki.model" "KMA tokenized text file"

from gensim.models import Word2Vec
import sys

def load_text(filename):
		# f = open(filename, "r", encoding='utf-8')
		f = open(filename, "r", encoding='cp949')
		text = f.readlines()
		f.close()
		return text

def load_model(modelname):
	model = FastText.load(modelname)
	return model

def sent2tokens(text):
		tokens = []
		for sent in text:
			tokens.append(sent.split())
		return tokens

def Word2Vec_train(sentences):
	print(f"\Word2Vec training for sentences.\n")

	model = Word2Vec(sentences, min_count=1)	# train model
	print(model)	# summarize the loaded model

	# summarize vocabulary
	# words = list(model.wv.vocab) # Error -- gensim OLD version
	words = model.wv.index_to_key
	print(words)

	# access vector for one word
	# print(model['sentence']) # ERROR -- gensim OLD version
	print(model.wv.get_vector('sentence'))
	
	# save model
	model.save('model.bin')
	
	# load model
	new_model = Word2Vec.load('model.bin')
	print(new_model)


#from pprint import pprint as print
from gensim.models.fasttext import FastText
from gensim.test.utils import datapath

def FastText_train_textfile(filename):
	print(f"\nFastText training for corpus_file <{filename}>.")

	model = FastText(vector_size=300)
	#model = FastText(vector_size=12, min_count=1, hs=1, negative=0, seed=42, workers=1, bucket=BUCKET)

	# build the vocabulary
	model.build_vocab(corpus_file=filename)

	# train the model
	model.train(corpus_file=filename, epochs=model.epochs, total_examples=model.corpus_count, total_words=model.corpus_total_words,)

	return model


def FastText_train_sentences(sentences):
	print(f"\nFastText training for sentences.")

	model = FastText(vector_size=300)
	#model = FastText(vector_size=12, min_count=1, hs=1, negative=0, seed=42, workers=1, bucket=BUCKET)

	# build the vocabulary
	model.build_vocab(sentences)

	# train the model
	model.train(sentences, total_examples=len(sentences), epochs=300)

	return model

# for updating a pre-trained FastText model
def FastText_update_train_sentences(model, sentences):
	print(f"\nFastText update training for sentences.")

	# model = FastText(vector_size=300)
	#model = FastText(vector_size=12, min_count=1, hs=1, negative=0, seed=42, workers=1, bucket=BUCKET)

	# build the vocabulary
	model.build_vocab(sentences, update=True)

	# train the model
	model.train(sentences, total_examples=len(sentences), epochs=300)

	return model


if __name__ == "__main__":
	modelname = sys.argv[1]
	filename = sys.argv[2]
	
	model = load_model(modelname)
	sentences = load_text(filename)
	
	# FastText update training for sentences
	token_list = sent2tokens(sentences)
	model = FastText_update_train_sentences(model, token_list)
	model_file = filename[:-14] + '-FastText-ADD.model'
	model.save(model_file)

	print(f"Newly updated model <{model_file}> was created!\n")

	# load model
	new_model = Word2Vec.load(model_file)
	print(new_model)
